/*
 *	Kernel.cpp - contain class method of kernel of the project 
 *  	
 *  
 *  Petr Kudyn (99226)
 *	PB009 - spring 2004
 *
 */
#include <vector> 
#include <stdlib.h>
#include <time.h>
#include <math.h>       // Header File for mathematical function
//#include <mmsystem.h>

#pragma hdrstop

#include "Kernel.h"
#include "main.h"

extern TGlobal *TheWorld;   //global object containing everything (almost:)
extern TGame *TheGame;      //information about game, levels, etc.
extern TCHAR Apath[MAX_PATH]; // program path
extern HWND	hWnd;		// Holds Our Window Handle
//*** class TObject - parent of any object in scene (letter, lights..) ***
//************************************************************************

//*** constructor -inicialization parametrs of object****
TObject::TObject(int _ID, eType _Type, GLfloat _x, GLfloat _y, GLfloat _z, GLfloat _w, 
	     GLfloat _rotx, GLfloat _roty,GLfloat _rotz,bool _e)
{
 ID= _ID; 
 type = _Type;
 pos[0]= _x; pos[1]= _y; pos[2]= _z;                         //set position
 pos[3]= _w;                                                //homogenou coordinate
 rot[0]= _rotx; rot[1]= _roty; rot[2]= _rotz;               //set rotation
 enabled= _e;																							 //visibilit1y
 
}


//*** class TLetter  ***
//**********************

//*** constructor -inicialization parametrs of object****
TLetter::TLetter(int _ID, eType _Type, GLfloat _x, GLfloat _y, GLfloat _z, GLfloat _w, //constructor
	     GLfloat _rotx, GLfloat _roty, GLfloat _rotz,bool _e,
         GLfloat _scalex, GLfloat _scaley, GLfloat _scalez,
		 int _ascii, GLfloat _r, GLfloat _g, GLfloat _b):
                   TObject(_ID, _Type, _x, _y, _z, _w, _rotx, _roty,_rotz, _e) //parent TObejct constructor
{
 ascii= _ascii;										   //set code 
 scale[0] = _scalex; scale[1] = _scaley; scale[2] = _scalez; //set scale
 color[0]= _r; color[1]= _g; color[2]= _b;             //set color
}


//*** class TSwitch ***
//***********************

//*** show or hide the objects ***
void TSwitch::Perform(void)
{
 *p_enabled=action; //show or hide the object
 stage=eDone;                                 //the function is done
}


//*** class TLineInterpolator ***
//*******************************

//*** interpolate parameters of object along a line ***
void TLineInterpolator::Perform(void)
{
 if(p_x!=NULL)
  *p_x+=dx;
 if(p_y!=NULL)
  *p_y+=dy;
 if(p_z!=NULL)
  *p_z+=dz;
 if(p_a!=NULL)
  *p_a+=da;


 if(TheWorld->getTime()>=end_time)
  stage=eDone;
}

//*** compute gains ***
void TLineInterpolator::Init(void)
{
 dx=move_x/(end_time-start_time+1);
 dy=move_y/(end_time-start_time+1);
 dz=move_z/(end_time-start_time+1);
 da=move_a/(end_time-start_time+1);
}

//*** set pointers to properties of object ***
void TLineInterpolator::SetPointers(GLfloat *_x,GLfloat *_y,GLfloat *_z,GLfloat *_a)
{
 p_x=_x;
 p_y=_y;
 p_z=_z;
 p_a=_a;
}

//*** class TBezierInterpolator ***
//*******************************

//*** set pointers to properties of object ***
void TBezierInterpolator::SetPointers(GLfloat *_x,GLfloat *_y,GLfloat *_z,GLfloat *_a)
{
 p_x=_x;
 p_y=_y;
 p_z=_z;
}

//*** inicializatioin ***
void TBezierInterpolator::Init(void)
{
 timeInc=1/float((end_time-start_time+1)); //time quantum for moving across line
 lineTime=timeInc;
 lastP[0]=0;  //set last point to current point
 lastP[1]=0;
 lastP[2]=0;
}

//*** interpolate parameters of object along a line ***
void TBezierInterpolator::Perform(void)
{

 for(int i=0; i<3; i++) //for each dimension
	{ 
   A[i]=P1[i]*lineTime; // construcion deCasteljau
	 B[i]=(P2[i]-P1[i])*lineTime + P1[i];
	 C[i]=(P3[i]-P2[i])*lineTime + P2[i];
	 D[i]=(B[i]-A[i])*lineTime + A[i];
   E[i]=(C[i]-B[i])*lineTime + B[i];
   F[i]=(E[i]-D[i])*lineTime + D[i];
   d[i]=F[i]-lastP[i];  //computer gain
   lastP[i]=F[i]; //set lastPoint to current point
 }

if(p_x!=NULL)
 *p_x+=d[0];
if(p_y!=NULL)
 *p_y+=d[1];
if(p_z!=NULL)
 *p_z+=d[2];

lineTime+=timeInc;

 if(TheWorld->getTime()>=end_time)
  stage=eDone;

}



//*** class TGlobal  ***
//**********************

//*** constructor ***
TGlobal::TGlobal() : time(0), objects(NULL), SshadeMode(GL_SMOOTH), functions(NULL)
{
  view= new TView; //create viewPort
  run=true;        //run the ainmation 
  GetClearPath(Apath);  //get program path
 
 fObjects[0]='\0';             //set object file path
 strcat(fObjects, Apath);
 strcat(fObjects, OBJECT_FILE_NAME);

 elights[0]=GL_LIGHT0;
 elights[1]=GL_LIGHT1;
 elights[2]=GL_LIGHT2;
 elights[3]=GL_LIGHT3;
 elights[4]=GL_LIGHT4;
 elights[5]=GL_LIGHT5;
 elights[6]=GL_LIGHT6;
 elights[7]=GL_LIGHT7;

 time=0;  //reset time

}

//*** destructor ***
TGlobal::~TGlobal()
{
if(objects!=NULL)  //delete objects
{
 for(int i=0; i<numObjects; i++)
	if(objects[i]!=NULL)
	  delete objects[i];

  delete[] objects; //free memory
}

if(functions!=NULL)  //delete functions
{
 for(int i=0; i<numFunctions; i++)
	if(functions[i]!=NULL)
	  delete functions[i];

  delete[] functions; //free memory
}

if(view!=NULL)
  delete view;

}
				   
//*** set shade model ***
void TGlobal::SetShadeModel(GLenum _sm)
{
 SshadeMode= _sm;
 glShadeModel(GL_SMOOTH);
};

//*** read one usable line from file *** 
void TGlobal::readStr(FILE *f,char *s)
{
 do
 { 
   fgets(s, MAX_LINE_LENGTH, f);// read line 
 } while ((s[0] == '/') || (s[0] == '\n'));  //while there is / (means comment) or enter
}

//*** get array index from object ID ***
int TGlobal::getIndex(int _ID)
{
 for(int i=0; i<numObjects; i++)
	if(objects[i]->ID==_ID)
	 return i;

	return 0;
}

//*** load object from file ***
bool TGlobal::LoadObject(char *file, int aster)
{
 FILE *fin;                         //pointer to file
 char lineBuff[MAX_LINE_LENGTH];   //buffer for reading 1 line
 eType typeBuff;                      //buffer for type of object



 if((fin=fopen(file, "rt"))==NULL)
 {    //open file for reading
   MessageBox(hWnd, "Can't open object file!","Error", MB_OK|MB_ICONWARNING|MB_APPLMODAL);
   PostQuitMessage(0);   // if file doesn't exist -> exit program
 }

 while(1)
 {
 //readStr(fin, lineBuff)  //load solid
//	if(lineBuff[0]=='.') //end of solids
	 break;
 }
 
 readStr(fin,lineBuff);   
 sscanf(lineBuff, "%d", &numObjects);  //read number of objects
 
 objects = new TObject* [numObjects];  //allocate array for objects

 for (int i=0; i<numObjects; i++)  //read objects parametrs
 {
  readStr(fin, lineBuff);
  sscanf(lineBuff, "%d", &typeBuff); //get type of object
  if(typeBuff==eLetter)              //if object is letter
  {
    objects[i] = new TLetter;
	sscanf(lineBuff, "%d%d%f%f%f%f%f%f%f%f%f%x%f%f%f%d",    //read letter parametrs
	   &objects[i]->type, &objects[i]->ID, &objects[i]->pos[0], &objects[i]->pos[1],
	   &objects[i]->pos[2],&objects[i]->rot[0],&objects[i]->rot[1],&objects[i]->rot[2],
       &static_cast<TLetter*>(objects[i])->scale[0], &static_cast<TLetter*>(objects[i])->scale[1],
	   &static_cast<TLetter*>(objects[i])->scale[2], &static_cast<TLetter*>(objects[i])->ascii, 
       &static_cast<TLetter*>(objects[i])->color[0],&static_cast<TLetter*>(objects[i])->color[1], 
	   &static_cast<TLetter*>(objects[i])->color[2],&objects[i]->enabled); 
  }
  else             //if object is light
  {
   objects[i] = new TLight;
   sscanf(lineBuff, "%d%d%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%f%d",
	    &objects[i]->type,&objects[i]->ID,&objects[i]->pos[0], &objects[i]->pos[1], 
		&objects[i]->pos[2],&objects[i]->pos[3],
	    &objects[i]->rot[0],&objects[i]->rot[1],&objects[i]->rot[2], 
		&static_cast<TLight*>(objects[i])->ambient[0],
		&static_cast<TLight*>(objects[i])->ambient[1],&static_cast<TLight*>(objects[i])->ambient[2],
		&static_cast<TLight*>(objects[i])->diffuse[0],
		&static_cast<TLight*>(objects[i])->diffuse[1],&static_cast<TLight*>(objects[i])->diffuse[2],
		&static_cast<TLight*>(objects[i])->specular[0],
		&static_cast<TLight*>(objects[i])->specular[1],&static_cast<TLight*>(objects[i])->specular[2],
		&static_cast<TLight*>(objects[i])->spotExponent,
		&static_cast<TLight*>(objects[i])->spotCutOff,&static_cast<TLight*>(objects[i])->attenuation[0],
		&static_cast<TLight*>(objects[i])->attenuation[1],&static_cast<TLight*>(objects[i])->attenuation[2],
		&static_cast<TLight*>(objects[i])->enabled);
  
	static_cast<TLight*>(objects[i])->ambient[3]=1;   //alfa value of lights is always one
	static_cast<TLight*>(objects[i])->specular[3]=1;
	static_cast<TLight*>(objects[i])->diffuse[3]=1;

	}
 }
 readStr(fin, lineBuff);
 sscanf(lineBuff, "%f%f%f%f",
	             &view->pos[0],&view->pos[1],&view->pos[2],//viewport
	             &view->angle);

view->SetLookAt();
 
view->up[0]=0;
view->up[1]=1;
view->up[2]=0;



 
 readStr(fin, lineBuff);
 sscanf(lineBuff, "%d",&numFunctions); //read number of function
 
 functions =  new TFunction* [numFunctions]; //create array of function
 
 int iBuff;                      // auxiliary variables
 for(int j=0; j<numFunctions; j++) //for each function
 {
   readStr(fin, lineBuff);
   sscanf(lineBuff, "%d",&iBuff);
	 
	 if(iBuff==0)      //if it is switch
	 { 
	   functions[j] = new TSwitch; //create new switch
		 sscanf(lineBuff, "%d%d%d%d",&functions[j]->type, &functions[j]->objectID,  //load switch parameters
		     	&functions[j]->start_time, &static_cast<TSwitch*>(functions[j])->action);
		 functions[j]->end_time=functions[j]->start_time; 
		 static_cast<TSwitch*>(functions[j])->p_enabled=&objects[getIndex(functions[j]->objectID)]->enabled; //set pointer
	   functions[j]->start_time*=TIME_CONST;        //time correction we set times in 1/10
     functions[j]->end_time*=TIME_CONST;
	 }
	 else if (iBuff>=1 && iBuff<=10)        //else if it is line interpolation
	 {
		functions[j] = new TLineInterpolator; //create new line interpolator
		sscanf(lineBuff, "%d%d%d%d%f%f%f%f",&functions[j]->type, &functions[j]->objectID, //load parameters
		     	&functions[j]->start_time, &functions[j]->end_time,
					&static_cast<TLineInterpolator*>(functions[j])->move_x,
					&static_cast<TLineInterpolator*>(functions[j])->move_y,
					&static_cast<TLineInterpolator*>(functions[j])->move_z,
					&static_cast<TLineInterpolator*>(functions[j])->move_a);
		
		functions[j]->start_time*=TIME_CONST;        //time correction we set times in 1/10
    functions[j]->end_time*=TIME_CONST;
		static_cast<TLineInterpolator*>(functions[j])->Init();                  //compute gains
		switch(iBuff)
		{
		case 1: //linear position interpolation for objects 
        static_cast<TLineInterpolator*>(functions[j])->SetPointers(&objects[getIndex(functions[j]->objectID)]->pos[0],//set pointers 
                                &objects[getIndex(functions[j]->objectID)]->pos[1], //4. is null because position
																&objects[getIndex(functions[j]->objectID)]->pos[2],NULL); //has only 3 items
		    break;
		case 2://linear rotation and direction interpolation for objects 
        static_cast<TLineInterpolator*>(functions[j])->SetPointers(&objects[getIndex(functions[j]->objectID)]->rot[0],//set pointers 
                                &objects[getIndex(functions[j]->objectID)]->rot[1], //4. is null because rotation
																&objects[getIndex(functions[j]->objectID)]->rot[2],NULL); //has only 3 items
		    break;
		case 3://linear color interpolation for letters 
        static_cast<TLineInterpolator*>(functions[j])->SetPointers(&static_cast<TLetter*>(objects[getIndex(functions[j]->objectID)])->color[0],//set pointers 
                                &static_cast<TLetter*>(objects[getIndex(functions[j]->objectID)])->color[1], //4. is null because color
																&static_cast<TLetter*>(objects[getIndex(functions[j]->objectID)])->color[2],NULL); //has only 3 items
		    break;
		case 4://linear ambient color interpolation for lights
		    static_cast<TLineInterpolator*>(functions[j])->SetPointers(&static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->ambient[0],//set pointers 
                                &static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->ambient[1], //4. is null because color
																&static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->ambient[2],NULL); //has only 3 items
		    break;
		case 5://linear diffuse color interpolation for lights
		   static_cast<TLineInterpolator*>(functions[j])->SetPointers(&static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->diffuse[0],//set pointers 
                                &static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->diffuse[1], //4. is null because color
															&static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->diffuse[2],NULL); //has only 3 items
		   break;
		case 6://linear specular color interpolation for lights
		    static_cast<TLineInterpolator*>(functions[j])->SetPointers(&static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->specular[0],//set pointers 
                                &static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->specular[1], //4. is null because color
																&static_cast<TLight*>(objects[getIndex(functions[j]->objectID)])->specular[2],NULL); //has only 3 items
		    break;
		case 7://linear scale interpolation for letters
		     static_cast<TLineInterpolator*>(functions[j])->SetPointers(&static_cast<TLetter*>(objects[getIndex(functions[j]->objectID)])->scale[0],//set pointers 
                                &static_cast<TLetter*>(objects[getIndex(functions[j]->objectID)])->scale[1], //4. is null because scale
																&static_cast<TLetter*>(objects[getIndex(functions[j]->objectID)])->scale[2],NULL); //has only 3 items
  		 break;
		}

	 }
   else if (iBuff>=11 && iBuff<=13)        //else if it is bezier interpolation
	 {
	   functions[j] = new TBezierInterpolator;
		 sscanf(lineBuff, "%d%d%d%d%f%f%f%f%f%f%f%f%f",&functions[j]->type, &functions[j]->objectID, //load parameters
		     	&functions[j]->start_time, &functions[j]->end_time,
					&static_cast<TBezierInterpolator*>(functions[j])->P1[0],
					&static_cast<TBezierInterpolator*>(functions[j])->P1[1],
					&static_cast<TBezierInterpolator*>(functions[j])->P1[2],
					&static_cast<TBezierInterpolator*>(functions[j])->P2[0],
					&static_cast<TBezierInterpolator*>(functions[j])->P2[1],
					&static_cast<TBezierInterpolator*>(functions[j])->P2[2],
					&static_cast<TBezierInterpolator*>(functions[j])->P3[0],
					&static_cast<TBezierInterpolator*>(functions[j])->P3[1],
					&static_cast<TBezierInterpolator*>(functions[j])->P3[2]);
		
		 functions[j]->start_time*=TIME_CONST;        //time correction we set times in 1/10
     functions[j]->end_time*=TIME_CONST;
	  
		 switch(iBuff)
		 {
		 case 11: //bezier interpolator for objects
		     static_cast<TBezierInterpolator*>(functions[j])->SetPointers(&objects[getIndex(functions[j]->objectID)]->pos[0],//set pointers 
                                &objects[getIndex(functions[j]->objectID)]->pos[1], //4. is null because position
																&objects[getIndex(functions[j]->objectID)]->pos[2],NULL); //has only 3 items
      		break;
		 }
		 static_cast<TBezierInterpolator*>(functions[j])->Init(); //initialization 
	 
	 }

	 functions[j]->stage=eBefore;                // common properties for each function
 }


 fclose(fin); //close the file

  generateAsteroids(aster);

return true;
}

//*** get path of program ***
void TGlobal::GetClearPath(LPSTR _Apath)
{
 pathLength=GetModuleFileName(NULL, _Apath, MAX_PATH);//get path including name of program

 for(int i=pathLength; i>0; i--)  //cut program name, keep only path (directories + "\")
   if(Apath[i]=='\\'){
      Apath[i+1]='\0';
	  pathLength=i;
	  return;
  }   

}

//*** scan functions,if there are any changes ***
void TGlobal::scanFunction(void)
{
  for (int i=0; i<numFunctions; i++) //for each function
	{
    if(functions[i]->stage==eDone)  //function is done
		  continue;
		if(functions[i]->start_time>time) //functions cant be perform yet
		  continue;
		if(functions[i]->start_time<=time && functions[i]->stage==eBefore) //star the fnctions now and set stage on eRunning
      functions[i]->stage=eRunning;
		functions[i]->Perform();            //performe the function
	}
	return;
}
//*** find out if there are samo collison between asteroids and kube ***
void TGlobal::ScanCollision(void)
{
 for(int i=0; i<asteroids.size()+1; i++) //for each cupple of asteroids compute distance
 {
  for(int j=i+1; j<asteroids.size(); j++)
	{   //if distance is so short compute new vectors
	 if(sqrt(pow(asteroids[i].pos[0] - asteroids[j].pos[0],2)+pow(asteroids[i].pos[1] - asteroids[j].pos[1],2)+ \
			 pow(asteroids[i].pos[2] - asteroids[j].pos[2],2))<= (asteroids[i].radius+asteroids[j].radius))
     ComputeCollision(&asteroids[i], &asteroids[j]);
	}
  if(sqrt(pow(asteroids[i].pos[0] - view->pos[0],2)+pow(asteroids[i].pos[1] - view->pos[1],2)+ \
		 pow(asteroids[i].pos[2] - view->pos[2],2))<= (asteroids[i].radius+view->radius))
     ComputeCollision(&asteroids[i], view);
 }
}

//*** compute collision between two asteroids ***
void TGlobal::ComputeCollision(TAsteroid *a1, TAsteroid *a2)
{
float sizeA1, sizeA2, newSize1, newSize2, spBuff1, spBuff2;

sizeA1= sqrt(pow(a1->speed[0],2)+pow(a1->speed[1],2)+pow(a1->speed[2],2));
sizeA2= sqrt(pow(a2->speed[0],2)+pow(a2->speed[1],2)+pow(a2->speed[2],2));


newSize2= (sizeA1+sizeA2)*a1->weight/(a1->weight+a2->weight); //multiplication constants
newSize1= (sizeA1+sizeA2)*a2->weight/(a1->weight+a2->weight);

for(int i=0; i<3; i++) //new vectors
{
 spBuff1=a1->speed[i];
 spBuff2=a2->speed[i];
 a1->speed[i]=(newSize1/sizeA2)*spBuff2;
 a2->speed[i]=(newSize2/sizeA1)*spBuff1;
}
//if two asteroid has the same direction, the more light will bounce
if(a1->speed[0]==a2->speed[0] && a1->speed[1]==a2->speed[1] && a1->speed[2]==a2->speed[2])
 if(a1->weight>a2->weight)
 {
  a2->speed[0]=-a2->speed[0];
  a2->speed[1]=-a2->speed[1];
  a2->speed[2]=-a2->speed[2];
 }
 else
 { 
  a1->speed[0]=-a1->speed[0];
  a1->speed[1]=-a1->speed[1];
  a1->speed[2]=-a1->speed[2];
 }

a1->move();
a2->move();


}

//*** compute collision between asteroid a cube ***
void TGlobal::ComputeCollision(TAsteroid *a1, TView *v)
{
 float origCenter[3];
 v->TranlaforToOrigin(a1, origCenter); 

 if (origCenter[0]-a1->radius <= (v->cubeSize/2) && 
	   origCenter[1]-a1->radius <= (v->cubeSize/2) &&
	   origCenter[2]-a1->radius <= (v->cubeSize/2) ) //if collision
 {
   TheGame->gameOver=true;

   run=0;
}


}


//*** Adjust a random number into limits***
float TGlobal::AdjustRandomNum(float min, float max, int number)
{
int num=number;  //intereger number 
float num2;

num= (num%(int((100*max)-(100*min)+1)))+(100*min); //multiply by 10 and then divide result we get a float number 

num2=float(num);

return num2/100;
}

//*** is selected position free ***
bool TGlobal::IsPosFree(float p0,float p1,float p2,float r)
{

for(int i=0; i<asteroids.size(); i++) //for each asteroid
 if( sqrt((p0-asteroids[i].pos[0])*(p0-asteroids[i].pos[0]) + //distance beetvenn centers
     (p1-asteroids[i].pos[1])*(p1-asteroids[i].pos[1]) +   //must be higher than sume of radiuses
		 (p2-asteroids[i].pos[2])*(p2-asteroids[i].pos[2])) < (r+asteroids[i].radius+1))
		 return false;

 return true;


}

//*** generate asteroids; ***
void TGlobal::generateAsteroids(int count) 
{
 TAsteroid astBuff;
 static int iStat=1;
 

 for(int i=0; i<count; i++) //for each asteroid
 {
   do{
    
		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random radius
    astBuff.radius=AdjustRandomNum(MIN_AST_SIZE,MAX_AST_SIZE,rand());
    iStat+=astBuff.radius;
		 
		srand((unsigned)::time(NULL)*(unsigned)iStat);  //get random X
    astBuff.pos[0]=AdjustRandomNum(space.x1+2*astBuff.radius,space.x2-2*astBuff.radius,rand());
    iStat+=astBuff.pos[0];
  
		srand((unsigned)::time(NULL)*(unsigned)iStat);    //get random Y
    astBuff.pos[1]=AdjustRandomNum(space.y1+2*astBuff.radius,space.y2-2*astBuff.radius,rand());
    iStat+=astBuff.pos[1];

		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random Z
   astBuff.pos[2]=AdjustRandomNum(space.z2+2*astBuff.radius,space.z1-astBuff.radius,rand());
    iStat+=astBuff.pos[2];

	 }
 while(!IsPosFree(astBuff.pos[0],astBuff.pos[1],astBuff.pos[2],astBuff.radius)); 
	
    srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random colorR
    astBuff.color[0]=AdjustRandomNum(MIN_R,MAX_R,rand());
    iStat+=astBuff.color[0];

		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random colorG
    astBuff.color[1]=AdjustRandomNum(MIN_G,MAX_G,rand());
    iStat+=astBuff.color[1];
		
		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random colorB
    astBuff.color[2]=AdjustRandomNum(MIN_B,MAX_B,rand());
    iStat+=astBuff.color[2];

    srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random speed X
    astBuff.speed[0]=AdjustRandomNum(MIN_SPEED,MAX_SPEED,rand());
    iStat+=astBuff.speed[0];
		
		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random speed Y
    astBuff.speed[1]=AdjustRandomNum(MIN_SPEED,MAX_SPEED,rand());
    iStat+=astBuff.speed[1];
		
		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random speed Z
    astBuff.speed[2]=AdjustRandomNum(MIN_SPEED,MAX_SPEED,rand());
    iStat+=astBuff.speed[2];

    
		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random rotation speed X
    astBuff.rotSpeed[0]=AdjustRandomNum(MIN_ROT,MAX_ROT,rand());
    iStat+=astBuff.rotSpeed[0];
		
		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random rotation speed Y
    astBuff.rotSpeed[1]=AdjustRandomNum(MIN_ROT,MIN_ROT,rand());
    iStat+=astBuff.rotSpeed[1];
		
		srand((unsigned)::time(NULL)*(unsigned)iStat);   //get random rotation speed Z
    astBuff.rotSpeed[2]=AdjustRandomNum(MIN_ROT,MIN_ROT,rand());
    iStat+=astBuff.rotSpeed[2];

		astBuff.weight=3*PI*pow(10*astBuff.radius,3)/4; //compute weight
    astBuff.rot[0]=0; astBuff.rot[1]=0; astBuff.rot[2]=0;

		if(astBuff.speed[0]==0)      //if speed=0 set defaut speed
		  astBuff.speed[0]=0.008f;
    if(astBuff.speed[1]==0)   
		  astBuff.speed[1]=0.008f;
    if(astBuff.speed[2]==0)   
		  astBuff.speed[2]=0.008f;
    
 asteroids.push_back(astBuff); //push asteroid into vector
 }
}

//**** move with all asteroids ****
void TGlobal::moveAsteroids(void)
{
 for(int i=0; i<asteroids.size(); i++)// for each asteroid
 {
  asteroids[i].move(); //move with asteroids
  space.KeepAstInside(&asteroids[i]); //keep asteroid inside limits
 }
}

//*** class TView  ***
//**********************

//*** set LookAt point from angle ***
void TView::SetLookAt(void)
{

eye[0]=pos[0]- sin((PI/180)*angle);  //x
eye[2]=pos[2]+ cos((PI/180)*angle);  //z
eye[1]=pos[1]+0.13;   //y

}

//*** rorationt by Y ***
void TView::Rotate(double a)
{
 angle+=a;    //increase angle
 if(angle>360 || angle<-360) //keep angle beetween -360 and +360 degrees
	 angle=0;


 SetLookAt(); //set look at point
}

//*** move front or back by Z
void TView::MoveFront(double step)
{
static posBuff;

posBuff=pos[0]+ step*sin((PI/180)*angle); //x

if(posBuff<TheWorld->space.x2 &&  posBuff>TheWorld->space.x1) //if is inside border
{
 pos[0]=pos[0]+ step*sin((PI/180)*angle);       //x
 pos[2]=pos[2]- step*cos((PI/180)*angle);  //z*/
}

SetLookAt();
}

//*** move up and down by Y ***
void TView::MoveUp(double step)
{
 static buffStep;
 buffStep=pos[1]+step;

 if(buffStep<TheWorld->space.y2 &&  buffStep>TheWorld->space.y1) //if is inside border
 {
  eye[1]+=step;
  pos[1]+=step;
 }
}

//**** increment or decrement step (horizontal speed)  ****
void TView::incStep(float inc)
{
if ((step+inc)<=MAX_STEP && (step+inc)>= -MAX_STEP) //speed is limited
 step+=inc;

}
//**** increment or decrement vertical step (vertical speed)  ****
void TView::incVerStep(float inc)
{
if ((verticalStep+inc)<=MAX_VERTICAL_STEP && (verticalStep+inc)>= -MAX_VERTICAL_STEP) //speed is limited
 verticalStep+=inc;


}
//tranlate asteroid to center when convex collision and computer exact collision
void TView::TranlaforToOrigin(TAsteroid *a1, float *nPos)
{
float buff0,buff2;

buff0=a1->pos[0]- pos[0];
nPos[1]=a1->pos[1]- pos[1];
buff2=a1->pos[2]- pos[2];

nPos[0]=buff0*cos((PI/180)*radius) - buff2*sin((PI/180)*radius);
nPos[2]=buff0*sin((PI/180)*radius) + buff2*cos((PI/180)*radius);

for(int i=0; i<3; i++)
 if(nPos[i]<0)  //absolute value
  nPos[i]=-nPos[i];

}


//*** class TSpace  ***
//**********************

//**** keep asteroids inside space, bounce them agains the virtual wall ***
void TSpace::KeepAstInside(TAsteroid *asteroid)
{

if(asteroid->pos[0]>=x2 || asteroid->pos[0]<=x1) //bounce against wall X 
{ 
 asteroid->speed[0]=-asteroid->speed[0];
 asteroid->move();
}
if(asteroid->pos[1]>=y2 || asteroid->pos[1]<=y1) //bounce against wall Y
{
 asteroid->speed[1]=-asteroid->speed[1];
 asteroid->move();
}

if(asteroid->pos[2]<= z2 || asteroid->pos[2]>= z1 )                   //bounce against wall Z and keep it near viewer
{ 
 asteroid->speed[2]=-asteroid->speed[2];
 asteroid->move();
}

}


//*** class TAsteroid ***
//***********************

//**** move the asteroid ***
void TAsteroid::move(void)
{
 for(int i=0; i<3; i++)  //for X,Z,Y
 {
  pos[i]+=speed[i];  //new position 
  rot[i]+=rotSpeed[i]; //new rotation
	if(rot[i]>360 || rot[i]<-360) //keep angle beetween -360 and +360 degrees
	 rot[i]=0;

 }
}

//*** class TGame ****
//***********************

//**** constructor *****
TGame::TGame(void)
{
 level=1;
 TheWorld = new TGlobal;  //create new world
 won=false;
 gameOver=false;
 overTime=50;
 numAsteroids[0]=10;  //numbers of asteroids in each level
 numAsteroids[1]=20;
 numAsteroids[2]=25;
 numAsteroids[3]=30;
 numAsteroids[4]=35;
 numAsteroids[5]=40;
 numAsteroids[6]=45;
 numAsteroids[7]=50;
 numAsteroids[8]=55;
 numAsteroids[9]=60;

 	  char buff[255];  //play the sound
   buff[0]='\0'; 

   strcat(buff,Apath);

	 strcat(buff,"data\\start.wav");
	 PlaySound(buff,NULL,SND_ASYNC);

}
//**** destructor *****
TGame::~TGame(void)
{
delete TheWorld;

} 

//*** move to next level, if it is last level then win ***
void TGame::NextLevel(void)
{
if(won)
 return;

if(level<NUM_LEVEL)
{
 level++;
 delete TheWorld;
 TheWorld= new TGlobal; //create new world
 InitGL();          //init gl and load object from file
 //show new level number, hide old level number
 TheWorld->objects[TheWorld->getIndex(FIRST_LEVEL_LIST+level-2)]->enabled=false;
 TheWorld->objects[TheWorld->getIndex(FIRST_LEVEL_LIST+level-1)]->enabled=true;
if(level==NUM_LEVEL) //show 10
 TheWorld->objects[TheWorld->getIndex(FIRST_LEVEL_LIST+level)]->enabled=true;
}
else //win
{
 win();
}
}

//*** scan if the velel is pass ***
void TGame::ScanLevel(void)
{
if (TheWorld->view->pos[2]<TheWorld->space.z2)
 NextLevel();

}

//*** if you win ***
void TGame::win(void)
{
 won=true;
 TheWorld->run=false; //stop the time

 TheWorld->view->pos[0]=-1;  //set possition before text
 TheWorld->view->pos[1]=100;
 TheWorld->view->pos[2]=-100;

 TheWorld->view->eye[0]=-1;
 TheWorld->view->eye[1]=100;
 TheWorld->view->eye[2]=8;

 for(int i=0;i<NUM_WIN ;i++) //enable text
   TheWorld->objects[TheWorld->getIndex(FIRST_WIN_ID+i)]->enabled=true;
 TheWorld->objects[TheWorld->getIndex(LIGHT_WIN)]->enabled=true;//enable light

 SetWindowText(hWnd,"Journey home - semestral project PB009,by Petr Kudyn (99226).  YOU WON!  CONGRATULATIONS!   Press F4 for new game."); 

  char buff[255];  //play the sound
   buff[0]='\0'; 

   strcat(buff,Apath);

	 if(TheWorld->getTime()%2)
	 strcat(buff,"data\\win.wav");
else
 strcat(buff,"data\\win2.wav");

	 PlaySound(buff,NULL,SND_ASYNC);


}
