/*
 *	Kernel.h - contain class definition of kernel of the project 
 *  	
 *  
 *  Petr Kudyn (99226)
 *	PB009 - spring 2004
 *
 */
#ifndef KERNELH
#define KERNELH


#include <vector> 

#pragma hdrstop

#include "main.h"

using namespace std;    //namespace for vector

class TLetter;
class TLight;
class TFunction;
class TPoint;
class TEdge;
class TAsteroid;


enum eType {eLetter=0, eLight};             //type of object
enum eStage {eBefore=0, eRunning, eDone};    //stage of function


//*** class TObject - objects in scene, virtual class ***
//*******************************************************
class TObject
{
public: //properties
 friend TLetter;
 friend TLight;
  
 GLfloat pos[4];                  //position
 GLfloat rot[3];                  //rotation
 eType type;                      //type of object
 bool enabled;                    //light can be switch on/off, letter visible/unvisible
 int ID;                          //unique ID 


public:  //function
  TObject(void){};
  TObject(int _ID, eType _Type, GLfloat _x, GLfloat _y, GLfloat _z, GLfloat _w, //constructor
	     GLfloat _rotx, GLfloat _roty,GLfloat _rotz, bool _e);
	virtual ~TObject() {};
};


//*** class TLetter - letters child of TObject***
//***********************************************
class TLetter : public TObject
{
public: //properties
 GLfloat scale[3]; //scale 
 GLfloat color[3];  //color 0=R, 1=G, 2=B
 
 int ascii; //ascii code of letter  /range = 32-126  in deciamal system

public: //functions
  TLetter(void){};
  TLetter(int _ID, eType _Type, GLfloat _x, GLfloat _y, GLfloat _z,GLfloat _w,  //constructor
	     GLfloat _rotx, GLfloat _roty, GLfloat _rotz,bool _e,
         GLfloat _scalex, GLfloat _scaley, GLfloat _scalez,
		 int _ascii, GLfloat _r, GLfloat _g, GLfloat _b);
	~TLetter() {};
};


//*** class TLight - lights child of TObject***
//*********************************************
class TLight : public TObject
{
public: //properties 
 GLfloat ambient[4], diffuse[4], specular[4]; //light color
 GLfloat spotExponent;                       //spot exponent
 GLfloat spotCutOff;                         //spotlight cutt of angle
 GLfloat attenuation[3];                     //attenuation factor 0-const, 1-linear, 2-quadratic
                             
public: //function
  TLight(void){};
  ~TLight() {};  

};
/*
//*** class TPoint ***
//********************
class TPoint
{
public: //properties
 GLfloat point[3], pointO[3];
//
};

//*** class TSolid - solid objects from triangular meshes***
//**********************************************************
class TSolid
{
public: //properties
 vector<TEdge> edges; //vector of appropriate edges
 vector<TEdge>::iterator eIterator;
  
 TPoint center; //center point actual and original
 GLfloat convRad[3]; //convex radius

 GLfloat translation[3]; //translation X,Y,Z
 GLfloat rotation[3];  //rotation X,Y,Z in degrees
 
 GLfloat speed[3]; //speed-spatial increment X,Y,Z per 1 time quantum
 GLfloat rotSpeed[3]; //rotation speed in degrees/1time quantum
};

//*** class TEdge ***
//*******************
class TEdge
{
public: //properties
  vector<TPoint> points;  //vector of appropriate poins
	vector<TPoint>::iterator pIterator;
  
	GLfloat normal[3];  //normal vector
};

*/



//*** class TFunction - transform functions, which can be applicate on objects, virtual class***
//**********************************************************************************************
class TFunction
{
public: //properties
int objectID;              //id of assigmented object
int type;                  //type of function
long start_time, end_time; //when action star/end
eStage stage;             //stage of action, can be before, now, done

public: //function
 virtual void Perform(void)=0;
};

//***class TSitch - can switch on/of single bool variable***
//**********************************************************
class TSwitch : public TFunction
{
public: //properties
bool action;              //type of action true=switch-on, false=switch-of

public: //function
 bool *p_enabled;            //pointer to switching variable
 void Perform(void);         //scan class, if the action should be perform
};


//***class TLineInterpolator - interpolate position, color, rotation, scale along the line
//*************************************************************************************
class TLineInterpolator: public TFunction
{
public: //properties

GLfloat dx, dy, dz, da;               //gain in all direction
GLfloat move_x,move_y,move_z,move_a;  //total move during all action
GLfloat *p_x,*p_y,*p_z,*p_a;                     //pointer to assigned properties of object

public: //functions
 TLineInterpolator(void):p_x(NULL),p_y(NULL),p_z(NULL),p_a(NULL){}; //constructor 
 void Init(void);																									 //compute gain from total move and time
 void SetPointers(GLfloat *_x,GLfloat *_y,GLfloat *_z,GLfloat *_a);//set pointer to object properties
 void Perform(void);																							 //perform interpolation
};

//***class TBezierInterpolator - interpolate position along the bezier cubic
//*************************************************************************************
class TBezierInterpolator: public TFunction
{
private: 
 GLfloat A[3],B[3],C[3],D[3],E[3],F[3]; //auxiliary point for computer line using DeCasteljau

public: //properties
GLfloat P1[3],P2[3],P3[3];              // control points, P0=[0,0,0]
GLfloat lastP[3], d[3];                 //last painted point
GLfloat *p_x,*p_y,*p_z;            //pointer to assigned properties of objec
GLfloat timeInc, lineTime;                        // 1/moving time, position on line

public: //functions
 TBezierInterpolator(void):p_x(NULL),p_y(NULL),p_z(NULL){};   //constructor  
 void SetPointers(GLfloat *_x,GLfloat *_y,GLfloat *_z,GLfloat *_a);    //set pointer to object properties
 void Perform(void);       //perform interpolation
 void Init(void);                                                     //inicialization
};

//*** class TView - information about viewport ***
//*************************************************
class TView
{
public: //properties

GLfloat eye[3];   //position of viewport
GLfloat pos[3];   //point we look at
GLfloat up[3];   //point whitch is up

GLfloat angle;  //rotation Y 

double step, verticalStep,angleInc;   //step length, angleIncrement
GLuint cubeList;                   //display list for borg cube 
float cubeSize;  //size of cube, lenght of edge

float radius;  //konvex envelope of cube


public: //function
 TView(void): step(0), angleInc(SLOW_ANGLE),verticalStep(0), cubeList(200), cubeSize(CUBE_SIZE)
 {radius=cubeSize*sqrt(3)/2;};

 void SetLookAt(void);  //set look at point from angle
 void Rotate(double a);   //rotate for angle a
 void MoveFront(double step);  //move front step>0 or back step<0
 void MoveUp(double step);
 void incStep(float inc);             //increment step(speed)
 void incVerStep(float inc);           //increment vertical step(speed)
 void TranlaforToOrigin(TAsteroid *a1, float *nPos); //tranlate asteroid to center when convex collision and computer exact collision
};


//*** TAsteroid - define asteroid properties ***
//**********************************************
class TAsteroid
{
public: //properties
 GLfloat pos[3];  //position X,Y,Z
 GLfloat speed[3]; //speed Z,Y,Z
 
 GLfloat rot[3];   //rotation X,Y,Z
 GLfloat rotSpeed[3]; //rotation speed X,Y,Z
 
 GLfloat color[3];  //color R,G,B
 GLfloat radius;    //radius of sphere
 GLfloat weight;   //weight 

public: //function

 void move(void);  //move the asteroid

};


//*** TSpace - define space with asteroid ***
//*******************************************
class TSpace
{
public:
 GLfloat x1,x2,y1,y2,z1,z2; //space size
 GLint borderList;  //display list for border
public:
 TSpace(void): x1(-4),x2(4),y1(-3),y2(3),z1(0),z2(-60), borderList(201){};
 void KeepAstInside(TAsteroid *asteroid);   //keep asteroid inside the space, bounce agains wall

};


//*** TGlobal- global setting, contain also poiter to array of all object ***
//***************************************************************************
class TGlobal
{
private: //properties
 GLenum SshadeMode;  					        // Shading
 int time;                            //actual time in timer1_period

public: //properties
 TObject **objects;                   //array contain each object in scene
 TFunction **functions;
 TView *view;                         //information about viewport
 int numObjects;                      //number of object in scene
 int numFunctions;                    //number of functions
 char fObjects[MAXPATH];              //file name, file with object definitions
 int pathLength;
 GLenum elights[8];                   //array of names of lights (GL_LIGHT0....GL_LIGHT7) 
 bool run;                            //1=animation is running, 0=pause
 
 vector<TAsteroid> asteroids;          //vector of asteroids 
 vector<TAsteroid>::iterator astIter;  //iterator for vector of asteroids


 TSpace space;                           //space with asteorid
 //vector<TSolid> solids;                //vector of solid objects


public: //function
 TGlobal(); //constructor
 ~TGlobal(); //destructor
 
 void SetShadeModel(GLenum _sm);   //set SshadeMode and call glShadeModel(_sm);
 GLenum getShadeModel(void){return SshadeMode;};       //get SshadeMode
 
 bool LoadObject(char *file,  int aster);      //load object from file to *objects
 void GetClearPath(LPSTR _Apath);  //get program path,cut program name, keep only path (directories + "\")
 void readStr(FILE *f,char *s);    //read one usable line from file 

 int getTime(void) {return time;};   //get current time
 void setTime(int t) {time=t;};      //set current time
 void incTime(void) {time++;};       //increment time

 
 int getIndex(int _ID);              //get array index from object ID
 void scanFunction(void);            //scan function for changes
 void generateAsteroids(int count);  //generate asteroids;
 float AdjustRandomNum(float min, float max, int number); //adjust random number in limits
 bool IsPosFree(float p0,float p1,float p2,float r); //is random selected asteroid position free?
 void moveAsteroids(void);  //move with all asteroids
 void ScanCollision(void);  //find out if there are samo collison between asteroids and kube
 void ComputeCollision(TAsteroid *a1, TAsteroid *a2); //compute collision asteroid-asteroid
 void ComputeCollision(TAsteroid *a1, TView *v); //compute collision asteroid-cube
};

//*** TGame - information about game, level etc. ***
//***************************************************************************
class TGame
{
public: //properties
 int level;     //actual level
 int numAsteroids[NUM_LEVEL]; //number of asteroids in levels
 bool won;                     //0=during a game, 1=if win
 bool gameOver;              //1=if you lose, otherewise 0
 int overTime;     //the new game starts a few second after crash defined by overTime
public:  //functions

 TGame(void); //constructor
 void NextLevel(void); //move to next level, if it is last level then win
 ~TGame(void); //destructor
 void ScanLevel(void);  //scan if we can switch level
 void win(void);                         //if you win
};



#endif

